package api

import (
	"baishuihu/db/model"
	"baishuihu/db/repo"
	"baishuihu/log"
	"errors"
	"strings"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func Banner(r *GinRouter) {
	r.POSTJWT("/banner/create", CreateBanner)
	r.POSTJWT("/banner/delete/:id", DeleteBanner)
	r.GET("/banner/list", BannerList)
	r.GET("/banner/detail/:id", BannerDetail)
	r.POSTJWT("/banner/update", UpdateBanner)
	r.POSTJWT("/banner/top", BannerTop)
}

func CreateBanner(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	collection := repo.CollectionBanner
	if strings.Contains(c.Request.URL.Path, "/nav") {
		collection = repo.CollectionNav
	}

	var banner model.Banner
	err := c.ShouldBindJSON(&banner)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	banner.CreateTime = time.Now()
	banner.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), collection, &banner)
}

func DeleteBanner(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	collection := repo.CollectionBanner
	if strings.Contains(c.Request.URL.Path, "/nav") {
		collection = repo.CollectionNav
	}
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), collection, _id)
}

func BannerList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	collection := repo.CollectionBanner
	if strings.Contains(c.Request.URL.Path, "/nav") {
		collection = repo.CollectionNav
	}

	page, size, query := UtilQueryPageSize(c)
	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: collection,
		Page:        page,
		Size:        size,
		Query:       query,
		Sort:        bson.D{bson.E{Key: "sort", Value: -1}, bson.E{Key: "createTime", Value: -1}},
	})
}

func BannerDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	collection := repo.CollectionBanner
	if strings.Contains(c.Request.URL.Path, "/nav") {
		collection = repo.CollectionNav
	}
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	cate := &model.Banner{}
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: collection,
		Query:       repo.Map{"_id": id},
	}, cate)
	if err != nil {
		log.Error(err)
		return nil, err
	}

	if !found {
		return nil, errors.New("未找到该数据")
	}
	return cate, nil
}

func UpdateBanner(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	collection := repo.CollectionBanner
	if strings.Contains(c.Request.URL.Path, "/nav") {
		collection = repo.CollectionNav
	}
	var cate model.Banner
	err := c.ShouldBindJSON(&cate)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if cate.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), collection, cate.Id.Hex(), &cate)
}

type BannerTopReq struct {
	Id primitive.ObjectID
}

// 置顶
func BannerTop(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var req BannerTopReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		return nil, errors.New("参数错误！")
	}

	banner := model.Banner{}
	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionBanner,
		Project:     []string{"sort"},
		Sort:        bson.M{"sort": -1},
	}, &banner)
	if err != nil {
		return nil, err
	}
	sort := 0
	if ok {
		sort = *banner.Sort
	}
	sort++
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionBanner, req.Id.Hex(), &model.Banner{Sort: &sort})
}
